#!/usr/bin/env python

from dsmapi2 import *

# Errorcode: 0: success (return parameters are valid); <0: Library error; >0: dSM error code
# library error codes: ds485-client-interface.h:
#     dsm error codes: dsm-api-v2/dsm-api-const.h

# client socket timeout: reset rxbuffer (prev: 117) [4]
# Traceback (most recent call last):
#     File "./dsmodel/structure.py", line 314, in <module>
#     for dsmid in fetch_all_dsmid(h):
#         File "./dsmodel/structure.py", line 283, in fetch_all_dsmid
#         raise createDsException(extractError(rec), "dSMInfo")
#     ds_exceptions.DsClientException: dSMInfo -1



# taken from: ds485-client-interface.h
ERROR_RESPONSE_TIMEOUT = -1,
NOTICE_CALL_TO_BROADCAST = -2,
ERROR_INVALID_HANDLE = -3,
ERROR_INVALID_CONNSPEC = -4,
ERROR_THREAD_START_FAILED = -5,
ERROR_NO_PORT_FOUND = -6,
ERROR_INVALID_ADDRESS = -7,
ERROR_CREATE_SOCKET = -8,
ERROR_SOCKET_OPTIONS = -9,
ERROR_SOCKET_CONNECT = -10,
ERROR_SOCKET_FULL = -11,
ERROR_SOCKET_FAILED = -12,
ERROR_SOCKET_VANISHED = -13,
ERROR_SOCKET_WOULD_BLOCK = -14, # seen in ean readout, after ctrl-c
ERROR_SOCKET_UNKNOWN = -15,
ERROR_INVALID_FD = -16,
ERROR_INVALID_PARAMETER = -17,
ERROR_REC_WRONG_FUNCTION_ID = -18,
ERROR_REC_WRONG_MODIFIER = -19,
ERROR_SYNC_RESPONSE_TIMEOUT = -20,
ERROR_DSID_CONVERSION = -21

class DsmApiErrorCodes:
    ERROR_OK = 0x00, #< No Error. Call could be executed. */
    ERROR_WRONG_PARAMETER = 0x01, #< A parameter is invalid. */
    ERROR_ZONE_NOT_FOUND = 0x02, #< The requested zone is not configured on this dSM. */
    ERROR_DEVICE_NOT_FOUND = 0x03, #< The requested device is not known on this dSM. */
    ERROR_GROUP_NOT_FOUND = 0x04, #< The requested group is not configured on this dSM. */
    ERROR_ZONE_CAN_NOT_BE_DELETED = 0x05, #< The zone is the last one configured and can not be deleted. */
    ERROR_GROUP_CAN_NOT_BE_DELETED = 0x06, #< The standard groups (colors) can not be deleted. */
    ERROR_DEVICE_CAN_NOT_BE_DELETED = 0x07, #< The device is locked and can not be deleted. */
    ERROR_NO_FURTHER_ZONES = 0x08, #< Maximal number of zones reached. */
    ERROR_NO_FURTHER_GROUPS = 0x09, #< None */
    ERROR_ZONE_ALREADY_EXISTS = 0x0a, #< A zone with this number is already configured on this dSM. */
    ERROR_GROUP_ALREADY_EXISTS = 0x0b, #< A group with this number is already configured on this dSM. */
    ERROR_ZONE_NOT_EMPTY = 0x0c, #< The zone could not be removed because it still contains devices. */
    ERROR_TIMEOUT = 0x0d, #< None */
    ERROR_WRONG_SIZE = 0x0e, #< The number of parameters is wrong. (Indicates mismatch between dsm-api library and dSM firmware.) */
    ERROR_WRONG_MSGID = 0x0f, #< The MessageId is unknown. (Indicates mismatch between dsm-api library and dSM firmware.) */
    ERROR_WRONG_MODIFIER = 0x10, #< The ModifierId is unknown. (Indicates mismatch between dsm-api library and dSM firmware.) */
    ERROR_WRONG_PACKET_NR = 0x11, #< Image transfer: received a wrong packet (not in sequence). */
    ERROR_WRONG_IMAGE_SIZE = 0x12, #< Image transfer: received more data than expected. */
    ERROR_NO_IMAGE_TRANSFER_ACTIVE = 0x13, #< Image transfer: received image packets with no transfer active. */
    ERROR_IMAGE_INVALID = 0x14, #< No valid software image found. */
    ERROR_NO_CONFIG = 0x15, #< No DSP configuration block available. */
    ERROR_REQUEST_CAN_NOT_BE_EXECUTED = 0x16, #< The request can not be executed. (e.g. DSP not ready) */
    ERROR_OUT_OF_RESOURCES = 0x17, #< The dSM has no resources to execute this request. */
    ERROR_PROGRAMMING_MODE_IS_DISABLED = 0x18, #< Programming mode is disabled (see dSM flags). *



# TODO, see DeviceBinaryInput_get_count, dSM Version Error, add check and
# warning when fetching data from dsm in structure handler

class DsException(Exception):
    def __init__(self, code, context_desc):
        Exception.__init__(self, context_desc + " %d" % code)

class DsClientException(DsException):
    def __init__(self, **args):
        DsException.__init__(self, **args)

# problems with: productlist_dsd.csv, productlist_dsm.csv, productlist_eth.csv
class DsDsidLookupException(DsClientException):
    def __init__(self, **args):
        DsClientException.__init__(self, **args)

def createDsClientException(**args):
    if args['code'] == ERROR_DSID_CONVERSION:
        return DsDsidLookupException(**args)
    else:
        return DsClientException(**args)


# protocol errors, aka device does not exist
class DsApiException(DsException):
    def __init__(self, **args):
        DsException.__init__(self, **args)

class DsApiDeviceNotFound(DsException):
    def __init__(self, **args):
        DsApiException.__init__(self, **args)


def createDsApiException(**args):
    code = args['code']
    if code == DsmApiErrorCodes.ERROR_DEVICE_NOT_FOUND:
        return DsApiDeviceNotFound(**args)
    else:
        return DsApiException(**args)


def createDsException(code, context_desc):
    args = dict(code = code, context_desc=context_desc)
    if code > 0:
        return DsApiException(**args)
    else:
        return createDsClientException(**args)


def extractError(ret):
    '''
    case type(ret):
        int: ret
        tuple: ret[0]
    '''
    if type(ret) == int:
        return ret
    elif type(ret) == tuple:
        return ret[0]
    else:
        assert(False)


if __name__=="__main__":
    try:
        createDsException(ERROR_RESPONSE_TIMEOUT, "DsmApiOpen failed")
    except DsClientException, e:
        print "catched: %s" % e

    # client errors, socket closed, table lookup fails
    createDsException(ERROR_DSID_CONVERSION, "my description")
    createDsException(ERROR_INVALID_HANDLE, "my description")

    # protocol errors
    createDsException(ERROR_DEVICE_NOT_FOUND, "my description")

    assert(extractError(1) == 1)
    assert(extractError([1, 0]) == 1)
    assert(extractError([0, 0]) == 0)
