#!/usr/bin/env python

from ds485d_mock import *
from ds485c_mock import *
from notification_queue import *
from process_pkt import *
from ds485.dsuid import *
from util import list2hexstr
from iptables import *
from testsuite_harness import *

from Queue import Queue, Empty
import unittest
import time, datetime
import sys, os
import traceback, signal
import logging
import random
from threading import Thread, Timer

TIMESTMP_FMT = ("%H:%M:%S") # no %f for msecs
FORMAT = '%(asctime)s.%(msecs)03d %(name)-5s %(message)s'
logging.basicConfig(format=FORMAT, datefmt=TIMESTMP_FMT, level=logging.INFO)

class Grack(unittest.TestCase):
    log = logging.getLogger("testsuite")

    def setUp(self):
        log = self.log
        #log.setLevel(logging.DEBUG)
        log.debug("setUp")
        log.debug("PID %d", os.getpid())
        signal.signal(signal.SIGINT, self.signal_handler)

        try:
            ports = [ 5004, 5005 ]
            self.ds485_port = 8945

            os.system("pkill -9 ds485p")
            iptables_unblock(log, ports[0])

            fork_ds485p(self.ds485_port, ports,
                        loglevel = 5, valgrind = True)

            self.select_loop = AsyncCore()
            self.select_loop.start()

            # open the socket after forking
            self.daemons = fire_up_daemons(ports)
            if not self.daemons:
                log.info("")
                log.info("ds485p failed to connected to our fake-buses")
                log.info("")
                seppuku()

            log.info("")
            log.info("ds485p connected to our fake-buses")
            log.info("")

        except Exception, e:
            print traceback.format_exc()
            raise e # TODO is tearDown called?


    def signal_handler(self, signal, frame):
        self.log.warning('You pressed Ctrl+C!')
        seppuku()

    def tearDown(self):
        self.log.info("tearDown")
        kill_ds485p()
        for d in self.daemons:
            d.close_all()
        self.select_loop.stop()

    def stop(self):
        self.log.info("do stop")
        self.run = False

    #@unittest.skip("")
    def test_server_reboot(self):
        log = self.log
        log.info("")
        log.info("Server reboot test")
        log.info("")
        log.info("We simulate reboot of a server, by shutting down one of our")
        log.info("our mock busses")
        log.info("The ds485 is sandwiched between two busses and one client.")
        log.info("Since there are two busses, the ds485p will not change it's bus")
        log.info("state. Hence the client sees only dsm 'left/joins' messages.")
        log.info("")

        dsmids = []
        for d in self.daemons:
            dsmids.extend(d.state.dsmids)

        class states:
            do_server_crash = "d"
            wait_dsm_left = "wd"
            do_server_reboot = "r"
            wait_dsm_joint = "wr"
            fetch_devices = "fd" # fetch devices after reconnect

        crashd_state = self.daemons[0].state
        self.run = True
        tstate = states.do_server_crash
        countdown = 10 # steady state
        client = None

        total_time = 3600 # 1 hour

        try:
            while self.run:
                try:
                    (evt, pkt, src) = notification_queue.get(True, 1)
                    reply = None
                except Empty: # timeout

                    countdown -= 1
                    if countdown > 0:
                        continue

                    total_time -= 1
                    if total_time <= 0:
                        self.assertTrue(False, "Total test time exceeded")

                    if not client:
                        log.info("Start Client")
                        client = Client('localhost', self.ds485_port, ClientState())
                        continue
                    else:
                        log.debug("client connected %d", client.state.connected)

                    cstate = client.state

                    if tstate == states.do_server_crash:
                        if not cstate.connected:
                            client.push(bytearray(createQueryBusState(0)))
                            continue

                        log.info("")
                        log.info("      simulate server crash")
                        log.info("")

                        for dsm in crashd_state.dsmids:
                            self.assertTrue(dsm in dsmids)

                        self.daemons[0].close_all()
                        del self.daemons[0]

                        tstate = states.wait_dsm_left

                    elif tstate == states.wait_dsm_left:
                        # wait till all dsms left
                        if (set(crashd_state.dsmids) & set(dsmids)):
                            continue

                        log.info("")
                        log.info("      server dsms left")
                        log.info("")

                        countdown = 50
                        tstate = states.do_server_reboot

                    elif tstate == states.do_server_reboot:

                        log.info("")
                        log.info("      simulate restart")
                        log.info("")

                        tstate = states.wait_dsm_joint
                        self.daemons.append(Daemon(crashd_state.port, crashd_state))

                    elif tstate == states.wait_dsm_joint:
                        if (set(crashd_state.dsmids) & set(dsmids) !=
                            set(crashd_state.dsmids)):
                                continue

                        countdown = 10
                        tstate = states.fetch_devices

                    elif tstate == states.fetch_devices:
                        self.run = False
                        log.info("")
                        log.info("              SUCCESS")
                        log.info("")


                #log.debug("evt: %s pkt: %s src: %r", evt,
                #          list2hexstr(pkt) if pkt else None, src)

                if isinstance(src, Connection):

                    if (evt == handle_read):
                        reply = process_pkt(pkt, log, src.state)

                elif isinstance(src, Client):

                    if (evt == handle_connect):
                        reply = createQueryBusState(0)
                    elif evt == handle_close:
                        log.info("server closed connection, reconnect")
                        client = None
                    elif (evt == handle_read):
                        # behave like a proper client
                        reply = process_pkt_client(pkt, log, src.state)

                        if pkt[0] == CMD.DS485D_CMD_BUS_CHANGE:
                            bc = parseBusChange(pkt)
                            if bc.joined and (bc.dsuid not in dsmids):
                                dsmids.append(bc.dsuid)
                            elif not bc.joined and (bc.dsuid in dsmids):
                                dsmids.remove(bc.dsuid)

                if not reply:
                    continue

                #log.info("reply: %s",  list2hexstr(reply))
                src.push(bytearray(reply))
                reply = None

            if client:
                client.close()

        except KeyboardInterrupt, e:
            print traceback.format_exc()
            pass
        except AssertionError, e:
            raise
        except Exception, e:
            print traceback.format_exc()
            pass

        log.info("exiting...")

if __name__== "__main__":
    unittest.main()
    #seppuku()
